/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#if !defined(AFX_BEMUSEDSERVERDLG_H__E44CC83D_4E15_4160_9EED_B03DC21C7D2C__INCLUDED_)
#define AFX_BEMUSEDSERVERDLG_H__E44CC83D_4E15_4160_9EED_B03DC21C7D2C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MAJOR_VERSION	1
#define MINOR_VERSION	73
#define PATCH_VERSION	""

#ifdef UNDER_CE
#define NUM_HANDLERS	2
#else
#define NUM_HANDLERS	5
#endif

#define TIMER_ID		1

#include "BemusedServer.h"

enum TStopMode
	{
	EImmediate,
	EFadeOut,
	EAtEndOfSong
	};

/////////////////////////////////////////////////////////////////////////////
// CDirTreeNode class

class CDirTreeNode
	{
public:
	CDirTreeNode();
	~CDirTreeNode();

	unsigned char iNodeType;
	unsigned char iFileType;
	TCHAR* iFileName;

	BOOL iWrittenToBuffer;

	CDirTreeNode* iParent;
	CDirTreeNode* iFirstChild;
	CDirTreeNode* iNextSibling;
	};


/////////////////////////////////////////////////////////////////////////////
// CAppHandler class


class CAppHandler
	{
public:
	virtual BOOL CheckRunning(const TCHAR* aPath, BOOL aStartAutomatically) = 0;
	virtual int GetVersion() = 0;
	virtual void GetInfo(unsigned char& aPlayStatus, int& aTotalTime, int& aCurrentTime,
		BOOL& aShuffle, BOOL& aRepeat, char* aSongNameGuess, int aSongNameGuessSize,
		BOOL aWaitForPlaying = TRUE) = 0;
	virtual void PlaySong(const char* aFileName) = 0;
	virtual void AddSongToPlaylist(const char* aFileName) = 0;
	virtual void SetVolume(int aVolume) = 0;
	virtual int GetVolume() = 0;
	virtual void Play() = 0;
	virtual void Pause() = 0;
	virtual void Stop(TStopMode aStopMode) = 0;
	virtual void FastForward() = 0;
	virtual void Rewind() = 0;
	virtual void NextTrack() = 0;
	virtual void PreviousTrack() = 0;
	virtual void SetShuffle(BOOL aShuffleOn) = 0;
	virtual void SetRepeat(BOOL aRepeatOn) = 0;
	virtual void GetPlaylist(char*& aPlaylist, int& aPlaylistLength, int& aPlaylistPos) = 0;
	virtual void SelectInPlaylist(int aIndex) = 0;
	virtual void RemoveAllFromPlaylist() = 0;
	virtual void OneSecondTick() = 0;
	virtual void GetDetailedInfo(int& aBitRate, int& aSampleRate, int& aChannels) = 0;
	virtual void FullScreen() = 0;
	virtual void SetSeek(int aSeek) = 0;
	virtual void GetPlaylistLength(int& aPlaylistLength) = 0;
	};


/////////////////////////////////////////////////////////////////////////////
// CBemusedServerDlg dialog

class CBemusedServerDlg : public CDialog
{
// Construction
public:
	CBemusedServerDlg(CWnd* pParent = NULL);	// standard constructor
	~CBemusedServerDlg();

	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);

// Dialog Data
	//{{AFX_DATA(CBemusedServerDlg)
	enum { IDD = IDD_BEMUSEDSERVER_DIALOG };
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBemusedServerDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	//{{AFX_MSG(CBemusedServerDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnReconnect();
	afx_msg void OnClose();
	afx_msg void OnExit();
	afx_msg void OnOK();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnBrowsePath();
	afx_msg void OnBrowseWinamp();
	afx_msg void OnAddpath();
	afx_msg void OnClearpath();
	afx_msg void OnSelchangeAppcombo();
	afx_msg void OnFindcomport();
	afx_msg void OnEndSession(BOOL bEnding);
	afx_msg void OnActivate(UINT nState, CWnd *pWndOther, BOOL bMinimized);
	afx_msg void OnStopwhenswitching();
	afx_msg void OnListenonsocket();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:
	int FindCommPortInRegistry();
	void InitialiseSettingsFromWizard(int aDefaultApplication, const TCHAR* aAppPath,
		const TCHAR* aMusicPath, const TCHAR* aCommPortName);

private:
	static UINT BluetoothThreadFunction(LPVOID pParam);
	static UINT SocketThreadFunction(LPVOID pParam);
	void BluetoothConnect(TCHAR* aCommPort);
	void ReadCommand(HANDLE aHandle, SOCKET aSocket = 0);
	void HandleBluetooth();
	void HandleSocket();

	BOOL CheckAppRunning();

	BOOL SetPrivilege(HANDLE hToken, LPCTSTR lpszPrivilege, BOOL bEnablePrivilege);

	BOOL ReadFromPhone(SOCKET aSocket, unsigned char* aBuffer, int aLength);

	void WriteListToPhone(SOCKET aSocket);
	void WriteDirectoryListToPhone(SOCKET aSocket);
	void WriteInfoToPhone(SOCKET aSocket);
	void WriteInf2ToPhone(SOCKET aSocket);
	void Play();
	void PlayFile(BOOL aAddToPlaylist, SOCKET aSocket);
	void GetFileInfo(SOCKET aSocket);
	void DownloadFile(SOCKET aSocket);
	void SetVolume(SOCKET aSocket);
	void GetVolume(SOCKET aSocket);
	void Stop(TStopMode aStopMode);
	void Pause();
	void NextTrack();
	void PreviousTrack();
	void FastForward();
	void Rewind();
	void Shuffle(SOCKET aSocket);
	void Repeat(SOCKET aSocket);
	void ShutDown();
	void SendAck(SOCKET aSocket);
	void WritePlaylist(SOCKET aSocket);
	void SelectInPlaylist(SOCKET aSocket);
	void RemoveAllFromPlaylist();
	void GetDetailedInfo(SOCKET aSocket);
	void FullScreen();
	void WriteVersion(SOCKET aSocket);

	void SetSeek(SOCKET aSocket);
	void WritePlaylistLength(SOCKET aSocket);

	void WriteIniFile();
	void ReadIniFile();

	void GenerateFileList(const TCHAR* aRoot, BOOL aRecursive);
	void ReadDir(TCHAR* aDir, CDirTreeNode* aNode, BOOL aRecursive);
	BOOL WriteNodeToBuffer(CDirTreeNode* aNode, unsigned char*& aBufferPtr, int& aSizeRemaining);
	void ResetWrittenFlag(CDirTreeNode* aNode);

	CAppHandler* FindHandlerForFile(const TCHAR* aFileName);

	void StopListeningOnSocket();
	void ListenOnSocket();

	void CopyIntIntoBuffer(int aValue, char* aBuffer);

	void Log(char* aFormatString, ...);

private:
	CDirTreeNode* iDirTreeRoot;
	CDirTreeNode* iDirTreeCurrentNode;

	unsigned char iCurrentVolume;

	HANDLE iBluetoothHandle;
	SOCKET iSocketHandle;
	BOOL iExitNow;
	int iErrorCount;
	CWinThread* iBluetoothThread;
	CWinThread* iSocketThread;

	BOOL iListenOnSocket;
	BOOL iClientClosed;

	SOCKADDR_IN iClientAddr;
	int iClientAddrLength;

	NOTIFYICONDATA iNotifyIconData;

	TCHAR iCommPortName[32];
	TCHAR iRootPath[MAX_PATH];
	TCHAR iStartDirectory[MAX_PATH];
	BOOL iHidden;

	CAppHandler* iAppHandler;
	CAppHandler* iAppHandlers[NUM_HANDLERS];
	TCHAR iAppPaths[NUM_HANDLERS][MAX_PATH];
	TCHAR iExtensionsHandled[NUM_HANDLERS][MAX_PATH];
	int iDefaultAppIndex;
	int iCurrentAppIndex;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BEMUSEDSERVERDLG_H__E44CC83D_4E15_4160_9EED_B03DC21C7D2C__INCLUDED_)
